@@ -86,7 +86,10 @@ SpectrumList_mzXMLImpl::SpectrumList_mzXMLImpl(shared_ptr<istream> is, const MSD
     }
 
     if (!gotIndex)
+    {
+        is_->clear();
         createIndex();
+    }
 
     scanMsLevelCache_.resize(index_.size());
 
@@ -171,27 +174,27 @@ struct HandlerPrecursor : public SAXParser::Handler
             if (activationMethod.empty() || activationMethod == "CID")
             {
                 // TODO: is it reasonable to assume CID if activation method is unspecified (i.e. older mzXMLs)?
-                precursor->activation.set(MS_CID);
+                precursor->activation.cvParams.emplace(precursor->activation.cvParams.begin(), MS_CID);
             }
             else if (activationMethod == "ETD")
-                precursor->activation.set(MS_ETD);
+                precursor->activation.cvParams.emplace(precursor->activation.cvParams.begin(), MS_ETD);
             else if (activationMethod == "ETD+SA")
             {
-                precursor->activation.set(MS_ETD);
-                precursor->activation.set(MS_CID);
+                precursor->activation.cvParams.emplace(precursor->activation.cvParams.begin(), MS_ETD);
+                precursor->activation.cvParams.emplace(precursor->activation.cvParams.begin(), MS_CID);
             }
             else if (activationMethod == "ECD")
-                precursor->activation.set(MS_ECD);
+                precursor->activation.cvParams.emplace(precursor->activation.cvParams.begin(), MS_ECD);
             else if (activationMethod == "HCD")
-                precursor->activation.set(MS_HCD);
+                precursor->activation.cvParams.emplace(precursor->activation.cvParams.begin(), MS_HCD);
             //else
                 // TODO: log about invalid attribute value
 
             if (!windowWideness.empty())
             {
                 double isolationWindowWidth = lexical_cast<double>(windowWideness) / 2.0;
-                precursor->isolationWindow.set(MS_isolation_window_lower_offset, isolationWindowWidth);
-                precursor->isolationWindow.set(MS_isolation_window_upper_offset, isolationWindowWidth);
+                precursor->isolationWindow.set(MS_isolation_window_lower_offset, isolationWindowWidth, MS_m_z);
+                precursor->isolationWindow.set(MS_isolation_window_upper_offset, isolationWindowWidth, MS_m_z);
             }
 
             if (!driftTime.empty())
@@ -200,7 +203,7 @@ struct HandlerPrecursor : public SAXParser::Handler
                 scan->set(MS_inverse_reduced_ion_mobility, invK0, MS_Vs_cm_2);
 
             if (!collisionalCrossSection.empty())
-                scan->userParams.emplace_back("CCS", collisionalCrossSection);
+                precursor->selectedIons.back().set(MS_collisional_cross_sectional_area, collisionalCrossSection, UO_square_angstrom);
 
             return Status::Ok;
         }
@@ -214,7 +217,8 @@ struct HandlerPrecursor : public SAXParser::Handler
         if (!precursor)
             throw runtime_error("[SpectrumList_mzXML::HandlerPrecursor] Null precursor."); 
 
-        precursor->selectedIons.back().set(MS_selected_ion_m_z, text, MS_m_z);
+        precursor->selectedIons.back().cvParams.emplace(precursor->selectedIons.back().cvParams.begin(), MS_selected_ion_m_z, text.c_str(), MS_m_z);
+        precursor->isolationWindow.cvParams.emplace(precursor->isolationWindow.cvParams.begin(), MS_isolation_window_target_m_z, text.c_str(), MS_m_z);
 
         return Status::Ok;
     }
@@ -485,16 +489,16 @@ class HandlerScan : public SAXParser::Handler
             if (endMz > 0)
                 scan.scanWindows.push_back(ScanWindow(startMz, endMz, MS_m_z));
             
-            if (!lowMz.empty())
-                spectrum_.set(MS_lowest_observed_m_z, lowMz);
-            if (!highMz.empty())
-                spectrum_.set(MS_highest_observed_m_z, highMz);
             if (!basePeakMz.empty())
-                spectrum_.set(MS_base_peak_m_z, basePeakMz);
+                spectrum_.set(MS_base_peak_m_z, basePeakMz, MS_m_z);
             if (!basePeakIntensity.empty())
-                spectrum_.set(MS_base_peak_intensity, basePeakIntensity);
+                spectrum_.set(MS_base_peak_intensity, basePeakIntensity, MS_number_of_detector_counts);
             if (!totIonCurrent.empty()) 
                 spectrum_.set(MS_total_ion_current, totIonCurrent);
+            if (!lowMz.empty())
+                spectrum_.set(MS_lowest_observed_m_z, lowMz, MS_m_z);
+            if (!highMz.empty())
+                spectrum_.set(MS_highest_observed_m_z, highMz, MS_m_z);
 
             return Status::Ok;
         }
@@ -566,7 +570,6 @@ class HandlerScan : public SAXParser::Handler
     bool getBinaryData_;
     string scanNumber_;
     string collisionEnergy_;
-    string activationMethod_;
     HandlerPeaks handlerPeaks_;
     HandlerPrecursor handlerPrecursor_;
     CVID nativeIdFormat_;
@@ -824,28 +827,36 @@ bool SpectrumList_mzXMLImpl::readIndex()
     if (indexIndexOffset == string::npos)
         return false; // no index present 
 
-    is_->seekg(-bufferSize + static_cast<int>(indexIndexOffset), std::ios::end);
-    if (!*is_)
-        throw index_not_found("[SpectrumList_mzXML::readIndex()] Error seeking to <indexOffset>."); 
-    
-    // read <indexOffset>
+    try
+    {
+        is_->seekg(-bufferSize + static_cast<int>(indexIndexOffset), std::ios::end);
+        if (!*is_)
+            throw index_not_found("[SpectrumList_mzXML::readIndex()] Error seeking to <indexOffset>."); 
 
-    stream_offset indexOffset = 0;
-    HandlerIndexOffset handlerIndexOffset(indexOffset);
-    SAXParser::parse(*is_, handlerIndexOffset);
-    if (indexOffset == 0)
-        throw index_not_found("[SpectrumList_mzXML::readIndex()] Error parsing <indexOffset>."); 
+        // read <indexOffset>
 
-    // read <index>
+        stream_offset indexOffset = 0;
+        HandlerIndexOffset handlerIndexOffset(indexOffset);
+        SAXParser::parse(*is_, handlerIndexOffset);
+        if (indexOffset == 0)
+            throw index_not_found("[SpectrumList_mzXML::readIndex()] Error parsing <indexOffset>."); 
 
-    is_->seekg(offset_to_position(indexOffset));
-    if (!*is_)
-        throw index_not_found("[SpectrumList_mzXML::readIndex()] Error seeking to <index>."); 
+        // read <index>
 
-    HandlerIndex handlerIndex(index_, msd_);
-    SAXParser::parse(*is_, handlerIndex);
-    if (index_.empty())
-        throw index_not_found("[SpectrumList_mzXML::readIndex()] <index> is empty.");
+        is_->seekg(offset_to_position(indexOffset));
+        if (!*is_)
+            throw index_not_found("[SpectrumList_mzXML::readIndex()] Error seeking to <index>."); 
+
+        HandlerIndex handlerIndex(index_, msd_);
+        SAXParser::parse(*is_, handlerIndex);
+        if (index_.empty())
+            throw index_not_found("[SpectrumList_mzXML::readIndex()] <index> is empty.");
+    }
+    catch (exception& e)
+    {
+        cerr << e.what() << endl; // TODO: log
+        return false;
+    }
 
     return true;
 }
